@@ -1,10 +1,12 @@
 __all__ = [
     "Run",
+    "NeptuneLoggingHandler",
 ]
 
 import warnings
 
 from neptune_scale.api.run import Run
 from neptune_scale.exceptions import NeptuneScaleWarning
+from neptune_scale.logging.logging_handler import NeptuneLoggingHandler
 
 warnings.simplefilter("once", category=NeptuneScaleWarning)
