@@ -1,7 +1,6 @@
 __all__ = [
     "Daemon",
     "get_logger",
-    "ProcessLink",
     "SharedFloat",
     "SharedInt",
     "SharedVar",
@@ -9,7 +8,6 @@ __all__ = [
 
 from neptune_scale.util.daemon import Daemon
 from neptune_scale.util.logger import get_logger
-from neptune_scale.util.process_link import ProcessLink
 from neptune_scale.util.shared_var import (
     SharedFloat,
     SharedInt,
